/* $Header: /home/cvs/bp/oofem/sm/src/ltrspace.h,v 1.8 2003/04/06 14:08:31 bp Exp $ */
/*

                   *****    *****   ******  ******  ***   ***                            
                 **   **  **   **  **      **      ** *** **                             
                **   **  **   **  ****    ****    **  *  **                              
               **   **  **   **  **      **      **     **                               
              **   **  **   **  **      **      **     **                                
              *****    *****   **      ******  **     **         
            
                                                                   
               OOFEM : Object Oriented Finite Element Code                 
                    
                 Copyright (C) 1993 - 2000   Borek Patzak                                       



         Czech Technical University, Faculty of Civil Engineering,
     Department of Structural Mechanics, 166 29 Prague, Czech Republic
                                                                               
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.                                                                              
*/

//   *******************************
//   *** CLASS LINEAR TETRAHEDRA ***
//   *******************************
#ifndef ltrspace_h

#include "structuralelement.h"
#include "fei3dtrlin.h"
#include "zznodalrecoverymodel.h"
#include "nodalaveragingrecoverymodel.h"
#include "sprnodalrecoverymodel.h"
#include "spatiallocalizer.h"
//


class LTRSpace : public StructuralElement, public ZZNodalRecoveryModelInterface, 
public NodalAveragingRecoveryModelInterface, public SPRNodalRecoveryModelInterface, 
public SpatialLocalizerInterface
//
{
 /*
   This class implements an tetrahedra four-node  elasticity finite element. 
   Each node has 3 degrees of freedom.
   
   DESCRIPTION :
   
   TASKS :
   
   - calculating its B,D,N matrices and dV.
   */
 
protected:
 static FEI3dTrLin interpolation;
 int numberOfGaussPoints;

public:
 LTRSpace (int,Domain*) ;             // constructor
  ~LTRSpace ()  {}                     // destructor

void               computeLumpedMassMatrix (FloatMatrix& answer, TimeStep* tStep) ;
void               computeMassMatrix (FloatMatrix& answer, TimeStep* tStep) 
 {computeLumpedMassMatrix(answer, tStep);}
      //FloatArray*        ComputeBodyLoadVectorAt (Load*, TimeStep*) ;

 virtual int            computeNumberOfDofs () {return 12;}
 virtual void           giveNodeDofIDMask  (int inode, IntArray& ) const;
  double             computeVolumeAround (GaussPoint*) ;
 /**
  Computes the global coordinates from given element's local coordinates.
  Required by nonlocal material models. 
  @returns nonzero if successful
  */
 virtual int computeGlobalCoordinates (FloatArray& answer, const FloatArray& lcoords) ;

 /**
  Computes the element local coordinates from given global coordinates.
  @returns nonzero if successful (point inside); zero otherwise
  */
 virtual int computeLocalCoordinates (FloatArray& answer, const FloatArray& gcoords);

 /** characteristic length in gp (for some material models) */
 double        giveCharacteristicLenght (GaussPoint* gp, const FloatArray &normalToCrackPlane);


  /** Interface requesting service */
  Interface* giveInterface (InterfaceType);
  virtual int testElementExtension (ElementExtension ext) 
  {return (((ext==Element_EdgeLoadSupport)||(ext==Element_SurfaceLoadSupport))?1:0);}
// 
// 
// definition & identification
//
 const char*  giveClassName () const { return "LTRSpace" ;}
 classType     giveClassID ()   const { return LTRSpaceClass; } 
 IRResultType initializeFrom (InputRecord* ir);
 MaterialMode          giveMaterialMode() {return   _3dMat;}
 Element_Geometry_Type giveGeometryType() const {return EGT_tetra_1;}

#ifdef __OOFEG
  void          drawRawGeometry (oofegGraphicContext&);
  void          drawDeformedGeometry(oofegGraphicContext&, UnknownType);
  virtual void  drawScalar   (oofegGraphicContext& context);
  virtual void  drawSpecial  (oofegGraphicContext&);
  //void          drawInternalState (oofegGraphicContext&);
#endif

/**
 @name The element interface required by ZZNodalRecoveryModel
*/
//@{
 /**
  Returns the size of DofManger record required to hold recovered values for given mode.
  @param type determines the type of internal variable to be recovered
  @return size of DofManger record required to hold recovered values
  */
  int ZZNodalRecoveryMI_giveDofManRecordSize(InternalStateType type);
 /**
  Returns the corresponding element to interface 
  */
  Element* ZZNodalRecoveryMI_giveElement () {return this;}
  /**
  Evaluates N matrix (interpolation estimated stress matrix).
  */
  void ZZNodalRecoveryMI_ComputeEstimatedInterpolationMtrx (FloatMatrix& answer, GaussPoint* aGaussPoint, 
                                    InternalStateType type);
//@}
/**
 @name The element interface required by NodalAveragingRecoveryModel
*/
//@{
 /**
  Computes the element value in given node.
  @param answer contains the result
  @param node element node number
  @param type determines the type of internal variable to be recovered
  @param tStep time step
  */
  void NodalAveragingRecoveryMI_computeNodalValue (FloatArray& answer, int node,
                          InternalStateType type, TimeStep* tStep);
 /**
  Computes the element value in given side.
  @param answer contains the result
  @param node element side number
  @param type determines the type of internal variable to be recovered
  @param tStep time step
  */
  void NodalAveragingRecoveryMI_computeSideValue (FloatArray& answer, int side,
                          InternalStateType type, TimeStep* tStep);
 /**
  Returns the size of DofManger record required to hold recovered values for given mode.
  @param type determines the type of internal variable to be recovered
  @return size of DofManger record required to hold recovered values
  */
 virtual int NodalAveragingRecoveryMI_giveDofManRecordSize(InternalStateType type) 
  {return ZZNodalRecoveryMI_giveDofManRecordSize(type);}
//@}
/**
 @name The element interface required by SPRNodalRecoveryModelInterface
*/
//@{
   void SPRNodalRecoveryMI_giveSPRAssemblyPoints (IntArray& pap);
   void SPRNodalRecoveryMI_giveDofMansDeterminedByPatch (IntArray& answer, int pap);
   int SPRNodalRecoveryMI_giveDofManRecordSize(InternalStateType type)  
   {return ZZNodalRecoveryMI_giveDofManRecordSize(type);}
  int SPRNodalRecoveryMI_giveNumberOfIP ();
  //void SPRNodalRecoveryMI_giveIPValue (FloatArray& answer, int ipNum, InternalStateType type);
  void SPRNodalRecoveryMI_computeIPGlobalCoordinates (FloatArray& coords, GaussPoint* gp);
  SPRPatchType SPRNodalRecoveryMI_givePatchType();
//@}

/**
 @name The element interface required by SpatialLocalizerInterface
*/
//@{
 /// Returns reference to corresponding element 
 virtual Element* SpatialLocalizerI_giveElement(){return this;}
 /// Returns nonzero if given element contains given point
 virtual int SpatialLocalizerI_containsPoint (const FloatArray& coords);
 /// Returns distance of given point from element parametric center
 virtual double SpatialLocalizerI_giveDistanceFromParametricCenter (const FloatArray& coords);
//@}

//

protected:
 void               computeBmatrixAt (GaussPoint*,FloatMatrix&,int=1,int=ALL_STRAINS) ;
  void               computeNmatrixAt (GaussPoint*, FloatMatrix&) ;
  void       computeGaussPoints () ;
 integrationDomain  giveIntegrationDomain () {return _Tetrahedra;}

/**
 @name Edge load support
*/
//@{
  void  computeEgdeNMatrixAt (FloatMatrix& answer, GaussPoint*) ;
  void  giveEdgeDofMapping (IntArray& answer, int) const;
  double        computeEdgeVolumeAround (GaussPoint*, int);
  void          computeEdgeIpGlobalCoords (FloatArray& answer, GaussPoint* gp, int iEdge) ;
  int   computeLoadLEToLRotationMatrix (FloatMatrix&, int, GaussPoint*) ;
 
//@}
/**
 @name Surface load support
*/
//@{
 virtual void computeSurfaceNMatrixAt (FloatMatrix& answer, GaussPoint*) ;
 virtual void  giveSurfaceDofMapping (IntArray& answer, int) const;
 virtual IntegrationRule* GetSurfaceIntegrationRule (int) ;
 virtual double        computeSurfaceVolumeAround (GaussPoint*, int) ;
 virtual void        computeSurfIpGlobalCoords (FloatArray& answer, GaussPoint*, int) ;
 virtual int  computeLoadLSToLRotationMatrix (FloatMatrix& answer, int, GaussPoint*) ;
//@}
} ;

#define ltrspace_h
#endif   








